<?php

declare(strict_types=1);

namespace Ig\IgGooglemaps\Command;

use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Http\RequestFactory;
use TYPO3\CMS\Core\Http\JsonResponse;

class GeoCodeCommand extends Command
{
     
    protected $api_url = 'https://maps.googleapis.com/maps/api/geocode/json?';
    
    protected function configure()
    {
        $this
            ->setDescription('')
            ->setHelp('')
            ->addArgument(
                'TableName',
                InputArgument::REQUIRED,
                'Tabellen Name'
            )
            
            ->addArgument(
                'Adress',
                InputArgument::OPTIONAL,
                'PLZ,ort,Strasse,StrassenNr,Land Spaltennamen in der Tabelle'
            )
            ->addArgument(
                'ids',
                InputArgument::OPTIONAL,
                'von id bis id (1,100)'
            )
            ->addArgument(
                'latlong',
                InputArgument::OPTIONAL,
                'lat,long Spaltennamen in der Tabelle'
            )
            ->addArgument(
                'maxvalues',
                InputArgument::OPTIONAL,
                'maximale anzahl an geocodes'
            );
    }


    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $io = new SymfonyStyle($input, $output);
        $extConf = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance(
                                                                        \TYPO3\CMS\Core\Configuration\ExtensionConfiguration::class
                                                                        )->get('ig_googlemaps');
                                                          
        if(isset($extConf['googleServerGeocodingApiKey']) && $extConf['googleServerGeocodingApiKey'] != null){
            $key = $extConf['googleServerGeocodingApiKey'];
        }
        else{
            $io->error('ig_googlemaps API Key wurde nicht gefunden');
            return 1;
        }

        $tableName = $input->getArgument('TableName');
        $latlong = $input->getArgument('latlong');
        $adress = $input->getArgument('Adress');
        $ids = $input->getArgument('ids');
        $maxvalues = intval($input->getArgument('maxvalues'));
        if(isset($latlong) && $latlong != null){
            $latlong = GeneralUtility::trimExplode(',' ,$latlong);
        }
        else{
            $latlong = [
                'latitude',
                'longitude'
            ];
        }
        
        if(isset($adress) && $adress != null){
             $adress = GeneralUtility::trimExplode(',' ,$adress);
        }
        else{
            $adress = [
                'plz',
                'ort',
                'strasse',
                'strassenNr',
                'land'
            ];
        }

        if(isset($ids) && $ids != null){
             $ids = GeneralUtility::intExplode(',' ,$ids);
        }
        else{
            $ids = [
                '0',
                '1',
            ];
        }
        
        
        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable($tableName);
            
        $result = $queryBuilder->select('*')
                                ->from($tableName)
                                ->where(
                                    $queryBuilder->expr()->gte(
                                        'id',
                                        $queryBuilder->createNamedParameter($ids[0], \PDO::PARAM_INT)
                                    ),

                                    $queryBuilder->expr()->lte(
                                        'id',
                                        $queryBuilder->createNamedParameter($ids[1] ?? $ids[0], \PDO::PARAM_INT)
                                    ),
                                    
                                    
                                    
                                    $queryBuilder->expr()->orX(
                                        $queryBuilder->expr()->isNull($latlong[0]),
                                        $queryBuilder->expr()->neq(
                                            $latlong[0], 
                                            $queryBuilder->createNamedParameter('')
                                        ),
                                        $queryBuilder->expr()->isNull($latlong[1]),
                                        $queryBuilder->expr()->neq(
                                            $latlong[1], 
                                            $queryBuilder->createNamedParameter('')
                                        ),
                                    )
                                )
                               ->setMaxResults($maxvalues)
                               ->executeQuery()
                               ->fetchAllAssociative();
        $addresses = [];
        $savedElements = 0;
        foreach ($result as $row) {
            $address = '';
            foreach($adress as $col){
                $address .= $row[$col] . ' ';
            }
                        
            $queryData = http_build_query(
                [
                    'key'     => $key,
                    'address' => $address,
                ]
            );
            $url = $this->api_url . $queryData;
            $res = GeneralUtility::makeInstance(RequestFactory::class)->request($url, 'GET', ['headers' => ['accept' => 'application/json']]);
            $content = json_decode($res->getBody()->getContents());
            
            $row[$latlong[0]] = $content->results[0]->geometry->location->lat;
            $row[$latlong[1]] = $content->results[0]->geometry->location->lng;
            //var_dump($row);die();
            if($row[$latlong[0]] != null && $row[$latlong[1]] != null){
                $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable($tableName);
                $queryBuilder->update($tableName)
                    ->where(
                        $queryBuilder->expr()->eq('id', $queryBuilder->createNamedParameter($row['id'], \PDO::PARAM_INT))
                    )
                    ->set($latlong[0], $row[$latlong[0]])
                    ->set($latlong[1], $row[$latlong[1]])
                    ->executeStatement();
                
                $savedElements++;
            }
        }

        if ($savedElements > 0) {
            $io->success('Es wurden ' . $savedElements . ' Elemente Gespeichert!');
            return 0;
        } else {
            $io->note('Es wurden Keine Elemente Gespeichert!');
            return 1;
        }
    }
}